@@ -257,7 +257,7 @@ SPIRVMap<Attribute::AttrKind, SPIRVFunctionControlMaskKind>::init() {
   add(Attribute::ReadOnly, FunctionControlConstMask);
   add(Attribute::AlwaysInline, FunctionControlInlineMask);
   add(Attribute::NoInline, FunctionControlDontInlineMask);
-  add(Attribute::OptimizeNone, FunctionControlOptNoneEXTMask);
+  add(Attribute::OptimizeNone, internal::FunctionControlOptNoneINTELMask);
 }
 typedef SPIRVMap<Attribute::AttrKind, SPIRVFunctionControlMaskKind>
     SPIRSPIRVFuncCtlMaskMap;
@@ -394,7 +394,6 @@ const static char ConvertHandleToImageINTEL[] = "ConvertHandleToImageINTEL";
 const static char ConvertHandleToSamplerINTEL[] = "ConvertHandleToSamplerINTEL";
 const static char ConvertHandleToSampledImageINTEL[] =
     "ConvertHandleToSampledImageINTEL";
-const static char InternalBuiltinPrefix[] = "__builtin_spirv_";
 } // namespace kSPIRVName
 
 namespace kSPIRVPostfix {
@@ -471,10 +470,11 @@ struct BuiltinArgTypeMangleInfo {
   bool IsLocalArgBlock;
   SPIR::TypePrimitiveEnum Enum;
   unsigned Attr;
+  PointerIndirectPair PointerElementType;
   BuiltinArgTypeMangleInfo()
       : IsSigned(true), IsVoidPtr(false), IsEnum(false), IsSampler(false),
         IsAtomic(false), IsLocalArgBlock(false), Enum(SPIR::PRIMITIVE_NONE),
-        Attr(0) {}
+        Attr(0), PointerElementType(nullptr, false) {}
 };
 
 /// Information for mangling builtin function.
@@ -483,92 +483,61 @@ public:
   /// Translate builtin function name and set
   /// argument attributes and unsigned args.
   BuiltinFuncMangleInfo(const std::string &UniqName = "")
-      : LocalArgBlockIdx(-1), VarArgIdx(-1), DontMangle(false) {
+      : VarArgIdx(-1), DontMangle(false) {
     if (!UniqName.empty())
       init(UniqName);
   }
   virtual ~BuiltinFuncMangleInfo() {}
   const std::string &getUnmangledName() const { return UnmangledName; }
-  void addUnsignedArg(int Ndx) { UnsignedArgs.insert(Ndx); }
+  void addUnsignedArg(int Ndx) {
+    if (Ndx == -1)
+      return addUnsignedArgs(0, 10); // 10 is enough for everybody, right?
+    getTypeMangleInfo(Ndx).IsSigned = false;
+  }
   void addUnsignedArgs(int StartNdx, int StopNdx) {
     assert(StartNdx < StopNdx && "wrong parameters");
     for (int I = StartNdx; I <= StopNdx; ++I)
       addUnsignedArg(I);
   }
-  void addVoidPtrArg(int Ndx) { VoidPtrArgs.insert(Ndx); }
-  void addSamplerArg(int Ndx) { SamplerArgs.insert(Ndx); }
-  void addAtomicArg(int Ndx) { AtomicArgs.insert(Ndx); }
-  void setLocalArgBlock(int Ndx) {
-    assert(0 <= Ndx && "it is not allowed to set less than zero index");
-    LocalArgBlockIdx = Ndx;
+  void addVoidPtrArg(unsigned Ndx) { getTypeMangleInfo(Ndx).IsVoidPtr = true; }
+  void addSamplerArg(unsigned Ndx) { getTypeMangleInfo(Ndx).IsSampler = true; }
+  void addAtomicArg(unsigned Ndx) { getTypeMangleInfo(Ndx).IsAtomic = true; }
+  void setLocalArgBlock(unsigned Ndx) {
+    getTypeMangleInfo(Ndx).IsLocalArgBlock = true;
   }
-  void setEnumArg(int Ndx, SPIR::TypePrimitiveEnum Enum) {
-    EnumArgs[Ndx] = Enum;
+  void setEnumArg(unsigned Ndx, SPIR::TypePrimitiveEnum Enum) {
+    auto &Info = getTypeMangleInfo(Ndx);
+    Info.IsEnum = true;
+    Info.Enum = Enum;
+  }
+  void setArgAttr(unsigned Ndx, unsigned Attr) {
+    getTypeMangleInfo(Ndx).Attr = Attr;
   }
-  void setArgAttr(int Ndx, unsigned Attr) { Attrs[Ndx] = Attr; }
   void setVarArg(int Ndx) {
     assert(0 <= Ndx && "it is not allowed to set less than zero index");
     VarArgIdx = Ndx;
   }
   void setAsDontMangle() { DontMangle = true; }
-  bool isArgUnsigned(int Ndx) {
-    return UnsignedArgs.count(-1) || UnsignedArgs.count(Ndx);
-  }
-  bool isArgVoidPtr(int Ndx) {
-    return VoidPtrArgs.count(-1) || VoidPtrArgs.count(Ndx);
-  }
-  bool isArgSampler(int Ndx) { return SamplerArgs.count(Ndx); }
-  bool isArgAtomic(int Ndx) { return AtomicArgs.count(Ndx); }
-  bool isLocalArgBlock(int Ndx) { return LocalArgBlockIdx == Ndx; }
-  bool isArgEnum(int Ndx, SPIR::TypePrimitiveEnum *Enum = nullptr) {
-    auto Loc = EnumArgs.find(Ndx);
-    if (Loc == EnumArgs.end())
-      Loc = EnumArgs.find(-1);
-    if (Loc == EnumArgs.end())
-      return false;
-    if (Enum)
-      *Enum = Loc->second;
-    return true;
-  }
   bool avoidMangling() { return DontMangle; }
-  unsigned getArgAttr(int Ndx) {
-    auto Loc = Attrs.find(Ndx);
-    if (Loc == Attrs.end())
-      Loc = Attrs.find(-1);
-    if (Loc == Attrs.end())
-      return 0;
-    return Loc->second;
-  }
   // get ellipsis index, single ellipsis at the end of the function is possible
   // only return value < 0 if none
   int getVarArg() const { return VarArgIdx; }
-  BuiltinArgTypeMangleInfo getTypeMangleInfo(int Ndx) {
-    BuiltinArgTypeMangleInfo Info;
-    Info.IsSigned = !isArgUnsigned(Ndx);
-    Info.IsVoidPtr = isArgVoidPtr(Ndx);
-    Info.IsEnum = isArgEnum(Ndx, &Info.Enum);
-    Info.IsSampler = isArgSampler(Ndx);
-    Info.IsAtomic = isArgAtomic(Ndx);
-    Info.IsLocalArgBlock = isLocalArgBlock(Ndx);
-    Info.Attr = getArgAttr(Ndx);
+  BuiltinArgTypeMangleInfo &getTypeMangleInfo(unsigned Ndx) {
+    while (Ndx >= ArgInfo.size())
+      ArgInfo.emplace_back();
+    BuiltinArgTypeMangleInfo &Info = ArgInfo[Ndx];
     return Info;
   }
   virtual void init(StringRef UniqUnmangledName) {
     UnmangledName = UniqUnmangledName.str();
   }
 
+  void fillPointerElementTypes(ArrayRef<PointerIndirectPair>);
+
 protected:
   std::string UnmangledName;
-  std::set<int> UnsignedArgs; // unsigned arguments, or -1 if all are unsigned
-  std::set<int> VoidPtrArgs;  // void pointer arguments, or -1 if all are void
-                              // pointer
-  std::set<int> SamplerArgs;  // sampler arguments
-  std::set<int> AtomicArgs;   // atomic arguments
-  std::map<int, SPIR::TypePrimitiveEnum> EnumArgs; // enum arguments
-  std::map<int, unsigned> Attrs;                   // argument attributes
-  int LocalArgBlockIdx; // index of a block with local arguments, idx < 0 if
-                        // none
-  int VarArgIdx;        // index of ellipsis argument, idx < 0 if none
+  std::vector<BuiltinArgTypeMangleInfo> ArgInfo;
+  int VarArgIdx; // index of ellipsis argument, idx < 0 if none
 private:
   bool DontMangle; // clang doesn't apply mangling for some builtin functions
                    // (i.e. enqueue_kernel)
@@ -612,7 +581,8 @@ void removeFnAttr(CallInst *Call, Attribute::AttrKind Attr);
 void addFnAttr(CallInst *Call, Attribute::AttrKind Attr);
 void saveLLVMModule(Module *M, const std::string &OutputFile);
 std::string mapSPIRVTypeToOCLType(SPIRVType *Ty, bool Signed);
-std::string mapLLVMTypeToOCLType(const Type *Ty, bool Signed);
+std::string mapLLVMTypeToOCLType(const Type *Ty, bool Signed,
+                                 Type *PointerElementType = nullptr);
 SPIRVDecorate *mapPostfixToDecorate(StringRef Postfix, SPIRVEntry *Target);
 
 /// Return vector V extended with poison elements to match the number of
@@ -700,7 +670,8 @@ StringRef dePrefixSPIRVName(StringRef R, SmallVectorImpl<StringRef> &Postfix);
 /// Get a canonical function name for a SPIR-V op code.
 std::string getSPIRVFuncName(Op OC, StringRef PostFix = "");
 
-std::string getSPIRVFuncName(Op OC, const Type *PRetTy, bool IsSigned = false);
+std::string getSPIRVFuncName(Op OC, const Type *PRetTy, bool IsSigned = false,
+                             Type *PointerElementType = nullptr);
 
 std::string getSPIRVFuncName(SPIRVBuiltinVariableKind BVKind);
 
@@ -723,7 +694,7 @@ Op getSPIRVFuncOC(StringRef Name, SmallVectorImpl<std::string> *Dec = nullptr);
 bool getSPIRVBuiltin(const std::string &Name, spv::BuiltIn &Builtin);
 
 /// \param Name LLVM function name
-/// \param DemangledName demangled name of the OpenCL built-in function
+/// \param DemangledName demanged name of the OpenCL built-in function
 /// \returns true if Name is the name of the OpenCL built-in function,
 /// false for other functions
 bool oclIsBuiltin(StringRef Name, StringRef &DemangledName, bool IsCpp = false);
@@ -800,12 +771,10 @@ CallInst *addCallInst(Module *M, StringRef FuncName, Type *RetTy,
                       StringRef InstName = SPIR_TEMP_NAME_PREFIX_CALL,
                       bool TakeFuncName = true);
 
-/// Check if an LLVM type is spirv.CooperativeMatrixKHR.
-bool isLLVMCooperativeMatrixType(llvm::Type *Ty);
-
 /// Add a call instruction for SPIR-V builtin function.
 CallInst *addCallInstSPIRV(Module *M, StringRef FuncName, Type *RetTy,
                            ArrayRef<Value *> Args, AttributeList *Attrs,
+                           ArrayRef<Type *> PointerElementTypes,
                            Instruction *Pos, StringRef InstName);
 
 /// Add a call of spir_block_bind function.
@@ -910,7 +879,8 @@ std::string getPostfix(Decoration Dec, unsigned Value = 0);
 /// Get postfix _R{ReturnType} for return type
 /// The returned postfix does not includ "_" at the beginning
 std::string getPostfixForReturnType(CallInst *CI, bool IsSigned = false);
-std::string getPostfixForReturnType(const Type *PRetTy, bool IsSigned = false);
+std::string getPostfixForReturnType(const Type *PRetTy, bool IsSigned = false,
+                                    Type *PointerElementType = nullptr);
 
 Constant *getScalarOrVectorConstantInt(Type *T, uint64_t V,
                                        bool IsSigned = false);
@@ -1037,6 +1007,7 @@ inline void getParameterTypes(CallInst *CI,
 /// manner
 std::string getSPIRVFriendlyIRFunctionName(OCLExtOpKind ExtOpId,
                                            ArrayRef<Type *> ArgTys,
+                                           ArrayRef<PointerIndirectPair> PETs,
                                            Type *RetTy = nullptr);
 
 /// Mangle a function in SPIR-V friendly IR manner
@@ -1050,6 +1021,7 @@ std::string getSPIRVFriendlyIRFunctionName(OCLExtOpKind ExtOpId,
 /// \return IA64 mangled name.
 std::string getSPIRVFriendlyIRFunctionName(const std::string &UniqName,
                                            spv::Op OC, ArrayRef<Type *> ArgTys,
+                                           ArrayRef<PointerIndirectPair> PETs,
                                            ArrayRef<SPIRVValue *> Ops);
 
 /// Cast a function to a void(void) funtion pointer.
@@ -1117,13 +1089,6 @@ bool lowerBuiltinVariableToCall(GlobalVariable *GV,
 // Transform all builtin variables into calls
 bool lowerBuiltinVariablesToCalls(Module *M);
 
-// Transform all builtin calls into variables
-bool lowerBuiltinCallsToVariables(Module *M);
-
-//  Transform all builtins into variables or calls
-//  depending on user specification
-bool lowerBuiltins(SPIRVModule *BM, Module *M);
-
 /// \brief Post-process OpenCL or SPIRV builtin function returning struct type.
 ///
 /// Some builtin functions are translated to SPIR-V instructions with
@@ -1152,167 +1117,6 @@ MetadataAsValue *map2MDString(LLVMContext &C, SPIRVValue *V);
 /// The return value is undefined if the input is larger than the largest power
 /// of two representable in SPIRVWord.
 [[nodiscard]] SPIRVWord bitCeil(SPIRVWord Value);
-
-/// \param MangledName LLVM function name.
-/// \param DemangledName demangled name of the input function if it is the
-/// translator's internal built-in function.
-/// \returns true if MangledName is the name of the translator's internal
-/// built-in function, false for other functions.
-/// Used for 'mini'-floats conversion functions
-bool isInternalSPIRVBuiltin(StringRef MangledName, std::string &DemangledName);
-
-// Wrapper around SPIR-V 1.6.4 FP Encoding to be used in the conversion
-// descriptor
-enum FPEncodingWrap {
-  Integer = FPEncoding::FPEncodingMax - 1,
-  IEEE754 = FPEncoding::FPEncodingMax,
-  BF16 = FPEncoding::FPEncodingBFloat16KHR,
-  E4M3 = FPEncoding::FPEncodingFloat8E4M3EXT,
-  E5M2 = FPEncoding::FPEncodingFloat8E5M2EXT,
-  E2M1 = internal::FPEncodingFloat4E2M1INTEL,
-};
-
-// Structure describing non-trivial conversions (FP8 and int4)
-struct FPConversionDesc {
-public:
-  FPEncodingWrap SrcEncoding;
-  FPEncodingWrap DstEncoding;
-  SPIRVWord ConvOpCode;
-
-  // To use as a key in std::map
-  bool operator==(const FPConversionDesc &Other) const {
-    return SrcEncoding == Other.SrcEncoding &&
-           DstEncoding == Other.DstEncoding && ConvOpCode == Other.ConvOpCode;
-  }
-
-  bool operator<(const FPConversionDesc &Other) const {
-    if (ConvOpCode != Other.ConvOpCode)
-      return ConvOpCode < Other.ConvOpCode;
-    if (SrcEncoding != Other.SrcEncoding)
-      return SrcEncoding < Other.SrcEncoding;
-    return DstEncoding < Other.DstEncoding;
-  }
-};
-
-// Maps internal builtin name to conversion descriptor
-typedef SPIRVMap<llvm::StringRef, FPConversionDesc> FPConvertToEncodingMap;
-
-// clang-format off
-template <> inline void FPConvertToEncodingMap::init() {
-  // 4-bit conversions
-  add("ConvertE2M1ToE4M3INTEL",
-      {FPEncodingWrap::E2M1,      FPEncodingWrap::E4M3,         OpFConvert});
-  add("ConvertE2M1ToE5M2INTEL",
-      {FPEncodingWrap::E2M1,      FPEncodingWrap::E5M2,         OpFConvert});
-  add("ConvertE2M1ToFP16INTEL",
-      {FPEncodingWrap::E2M1,      FPEncodingWrap::IEEE754,      OpFConvert});
-  add("ConvertE2M1ToBF16INTEL",
-      {FPEncodingWrap::E2M1,      FPEncodingWrap::BF16,         OpFConvert});
-
-  add("ConvertInt4ToE4M3INTEL",
-      {FPEncodingWrap::Integer,      FPEncodingWrap::E4M3,      OpConvertSToF});
-  add("ConvertInt4ToE5M2INTEL",
-      {FPEncodingWrap::Integer,      FPEncodingWrap::E5M2,      OpConvertSToF});
-  add("ConvertInt4ToFP16INTEL",
-      {FPEncodingWrap::Integer,      FPEncodingWrap::IEEE754,   OpConvertSToF});
-  add("ConvertInt4ToBF16INTEL",
-      {FPEncodingWrap::Integer,      FPEncodingWrap::BF16,      OpConvertSToF});
-  add("ConvertInt4ToInt8INTEL",
-      {FPEncodingWrap::Integer,      FPEncodingWrap::Integer,   OpSConvert});
-
-  add("ConvertFP16ToE2M1INTEL",
-      {FPEncodingWrap::IEEE754,      FPEncodingWrap::E2M1,      OpFConvert});
-  add("ConvertBF16ToE2M1INTEL",
-      {FPEncodingWrap::BF16,         FPEncodingWrap::E2M1,      OpFConvert});
-  add("ConvertFP16ToInt4INTEL",
-      {FPEncodingWrap::IEEE754,      FPEncodingWrap::Integer,   OpConvertFToS});
-  add("ConvertBF16ToInt4INTEL",
-      {FPEncodingWrap::BF16,         FPEncodingWrap::Integer,   OpConvertFToS});
-
-  // 8-bit conversions
-  add("ConvertE4M3ToFP16EXT",
-      {FPEncodingWrap::E4M3,         FPEncodingWrap::IEEE754,      OpFConvert});
-  add("ConvertE5M2ToFP16EXT",
-      {FPEncodingWrap::E5M2,         FPEncodingWrap::IEEE754,      OpFConvert});
-  add("ConvertE4M3ToBF16EXT",
-      {FPEncodingWrap::E4M3,         FPEncodingWrap::BF16,         OpFConvert});
-  add("ConvertE5M2ToBF16EXT",
-      {FPEncodingWrap::E5M2,         FPEncodingWrap::BF16,         OpFConvert});
-  add("ConvertFP16ToE4M3EXT",
-      {FPEncodingWrap::IEEE754,      FPEncodingWrap::E4M3,         OpFConvert});
-  add("ConvertFP16ToE5M2EXT",
-      {FPEncodingWrap::IEEE754,      FPEncodingWrap::E5M2,         OpFConvert});
-  add("ConvertBF16ToE4M3EXT",
-      {FPEncodingWrap::BF16,         FPEncodingWrap::E4M3,         OpFConvert});
-  add("ConvertBF16ToE5M2EXT",
-      {FPEncodingWrap::BF16,         FPEncodingWrap::E5M2,         OpFConvert});
-
-  // SPV_INTEL_fp_conversions
-  add("ClampConvertFP16ToE2M1INTEL",
-      {FPEncodingWrap::IEEE754,      FPEncodingWrap::E2M1,
-       internal::OpClampConvertFToFINTEL});
-  add("ClampConvertBF16ToE2M1INTEL",
-      {FPEncodingWrap::BF16,         FPEncodingWrap::E2M1,
-       internal::OpClampConvertFToFINTEL});
-  add("ClampConvertFP16ToE4M3INTEL",
-      {FPEncodingWrap::IEEE754,      FPEncodingWrap::E4M3,
-       internal::OpClampConvertFToFINTEL});
-  add("ClampConvertBF16ToE4M3INTEL",
-      {FPEncodingWrap::BF16,         FPEncodingWrap::E4M3,
-       internal::OpClampConvertFToFINTEL});
-  add("ClampConvertFP16ToE5M2INTEL",
-      {FPEncodingWrap::IEEE754,      FPEncodingWrap::E5M2,
-       internal::OpClampConvertFToFINTEL});
-  add("ClampConvertBF16ToE5M2INTEL",
-      {FPEncodingWrap::BF16,         FPEncodingWrap::E5M2,
-       internal::OpClampConvertFToFINTEL});
-  add("ClampConvertFP16ToInt4INTEL",
-      {FPEncodingWrap::IEEE754,      FPEncodingWrap::Integer,
-       internal::OpClampConvertFToSINTEL});
-  add("ClampConvertBF16ToInt4INTEL",
-      {FPEncodingWrap::BF16,         FPEncodingWrap::Integer,
-       internal::OpClampConvertFToSINTEL});
-
-  add("StochasticRoundFP16ToE5M2INTEL",
-      {FPEncodingWrap::IEEE754,      FPEncodingWrap::E5M2,
-       internal::OpStochasticRoundFToFINTEL});
-  add("StochasticRoundFP16ToE4M3INTEL",
-      {FPEncodingWrap::IEEE754,      FPEncodingWrap::E4M3,
-       internal::OpStochasticRoundFToFINTEL});
-  add("StochasticRoundBF16ToE5M2INTEL",
-      {FPEncodingWrap::BF16,         FPEncodingWrap::E5M2,
-       internal::OpStochasticRoundFToFINTEL});
-  add("StochasticRoundBF16ToE4M3INTEL",
-      {FPEncodingWrap::BF16,         FPEncodingWrap::E4M3,
-       internal::OpStochasticRoundFToFINTEL});
-  add("StochasticRoundFP16ToE2M1INTEL",
-      {FPEncodingWrap::IEEE754,      FPEncodingWrap::E2M1,
-       internal::OpStochasticRoundFToFINTEL});
-  add("StochasticRoundBF16ToE2M1INTEL",
-      {FPEncodingWrap::BF16,         FPEncodingWrap::E2M1,
-       internal::OpStochasticRoundFToFINTEL});
-  add("ClampStochasticRoundFP16ToInt4INTEL",
-      {FPEncodingWrap::IEEE754,      FPEncodingWrap::Integer,
-       internal::OpClampStochasticRoundFToSINTEL});
-  add("ClampStochasticRoundBF16ToInt4INTEL",
-      {FPEncodingWrap::BF16,         FPEncodingWrap::Integer,
-       internal::OpClampStochasticRoundFToSINTEL});
-
-  add("ClampStochasticRoundFP16ToE5M2INTEL",
-      {FPEncodingWrap::IEEE754,      FPEncodingWrap::E5M2,
-       internal::OpClampStochasticRoundFToFINTEL});
-  add("ClampStochasticRoundFP16ToE4M3INTEL",
-      {FPEncodingWrap::IEEE754,      FPEncodingWrap::E4M3,
-       internal::OpClampStochasticRoundFToFINTEL});
-  add("ClampStochasticRoundBF16ToE5M2INTEL",
-      {FPEncodingWrap::BF16,         FPEncodingWrap::E5M2,
-       internal::OpClampStochasticRoundFToFINTEL});
-  add("ClampStochasticRoundBF16ToE4M3INTEL",
-      {FPEncodingWrap::BF16,         FPEncodingWrap::E4M3,
-       internal::OpClampStochasticRoundFToFINTEL});
-}
-
-// clang-format on
 } // namespace SPIRV
 
 #endif // SPIRV_SPIRVINTERNAL_H
