@@ -38,11 +38,6 @@
 ///
 //===----------------------------------------------------------------------===//
 
-// This file needs to be included before anything that declares
-// llvm::PointerType to avoid a compilation bug on MSVC.
-#include "llvm/Demangle/Demangle.h"
-#include "llvm/Demangle/ItaniumDemangle.h"
-
 #include "FunctionDescriptor.h"
 #include "ManglingUtils.h"
 #include "NameMangleAPI.h"
@@ -55,6 +50,7 @@
 
 #include "llvm/ADT/StringSwitch.h"
 #include "llvm/Bitcode/BitcodeWriter.h"
+#include "llvm/Demangle/Demangle.h"
 #include "llvm/IR/IRBuilder.h"
 #include "llvm/IR/IntrinsicInst.h"
 #include "llvm/IR/Metadata.h"
@@ -69,21 +65,6 @@
 
 #define DEBUG_TYPE "spirv"
 
-namespace {
-
-// Return the value for when the dimension index of a builtin is out of range.
-uint64_t getBuiltinOutOfRangeValue(StringRef VarName) {
-  assert(VarName.startswith("__spirv_BuiltIn"));
-  return StringSwitch<uint64_t>(VarName)
-      .EndsWith("GlobalSize", 1)
-      .EndsWith("NumWorkgroups", 1)
-      .EndsWith("WorkgroupSize", 1)
-      .EndsWith("EnqueuedWorkgroupSize", 1)
-      .Default(0);
-}
-
-} // anonymous namespace
-
 namespace SPIRV {
 
 #ifdef _SPIRV_SUPPORT_TEXT_FMT
@@ -139,7 +120,7 @@ void saveLLVMModule(Module *M, const std::string &OutputFile) {
   Out.keep();
 }
 
-std::string mapLLVMTypeToOCLType(const Type *Ty, bool Signed) {
+std::string mapLLVMTypeToOCLType(const Type *Ty, bool Signed, Type *PET) {
   if (Ty->isHalfTy())
     return "half";
   if (Ty->isFloatTy())
@@ -184,6 +165,11 @@ std::string mapLLVMTypeToOCLType(const Type *Ty, bool Signed) {
   // value of some SPIR-V instructions may be represented as pointer to a struct
   // in LLVM IR) we can mangle the type.
   BuiltinFuncMangleInfo MangleInfo;
+  if (Ty->isPointerTy()) {
+    assert(cast<PointerType>(const_cast<Type *>(Ty))
+               ->isOpaqueOrPointeeTypeMatches(PET));
+    MangleInfo.getTypeMangleInfo(0).PointerElementType.setPointer(PET);
+  }
   std::string MangledName =
       mangleBuiltin("", const_cast<Type *>(Ty), &MangleInfo);
   // Remove "_Z0"(3 characters) from the front of the name
@@ -344,23 +330,6 @@ bool isSYCLBfloat16Type(llvm::Type *Ty) {
   return false;
 }
 
-bool isLLVMCooperativeMatrixType(llvm::Type *Ty) {
-  // In LLVM 14/15, TargetExtType doesn't exist, so cooperative matrices are
-  // represented as struct types with names like
-  // "spirv.CooperativeMatrixKHR._..."
-  auto *ST = dyn_cast<StructType>(Ty);
-  if (!ST || !ST->hasName())
-    return false;
-
-  StringRef STName = ST->getName();
-  if (!STName.startswith(kSPIRVTypeName::PrefixAndDelim))
-    return false;
-
-  SmallVector<std::string, 8> Postfixes;
-  std::string TN = decodeSPIRVTypeName(STName, Postfixes);
-  return TN == kSPIRVTypeName::CooperativeMatrixKHR;
-}
-
 Function *getOrCreateFunction(Module *M, Type *RetTy, ArrayRef<Type *> ArgTypes,
                               StringRef Name, BuiltinFuncMangleInfo *Mangle,
                               AttributeList *Attrs, bool TakeName) {
@@ -457,9 +426,10 @@ std::string getSPIRVFuncName(Op OC, StringRef PostFix) {
   return prefixSPIRVName(getName(OC) + PostFix.str());
 }
 
-std::string getSPIRVFuncName(Op OC, const Type *PRetTy, bool IsSigned) {
+std::string getSPIRVFuncName(Op OC, const Type *PRetTy, bool IsSigned,
+                             Type *PET) {
   return prefixSPIRVName(getName(OC) + kSPIRVPostfix::Divider +
-                         getPostfixForReturnType(PRetTy, IsSigned));
+                         getPostfixForReturnType(PRetTy, IsSigned, PET));
 }
 
 std::string getSPIRVFuncName(SPIRVBuiltinVariableKind BVKind) {
@@ -517,9 +487,10 @@ std::string getPostfixForReturnType(CallInst *CI, bool IsSigned) {
   return getPostfixForReturnType(CI->getType(), IsSigned);
 }
 
-std::string getPostfixForReturnType(const Type *PRetTy, bool IsSigned) {
+std::string getPostfixForReturnType(const Type *PRetTy, bool IsSigned,
+                                    Type *PET) {
   return std::string(kSPIRVPostfix::Return) +
-         mapLLVMTypeToOCLType(PRetTy, IsSigned);
+         mapLLVMTypeToOCLType(PRetTy, IsSigned, PET);
 }
 
 // Enqueue kernel, kernel query, pipe and address space cast built-ins
@@ -558,7 +529,7 @@ bool getSPIRVBuiltin(const std::string &OrigName, spv::BuiltIn &B) {
   return getByName(R.str(), B);
 }
 
-// DemangledName is a substring of Name. The DemangledName is updated only
+// Demangled name is a substring of the name. The DemangledName is updated only
 // if true is returned
 bool oclIsBuiltin(StringRef Name, StringRef &DemangledName, bool IsCpp) {
   if (Name == "printf") {
@@ -597,65 +568,6 @@ bool oclIsBuiltin(StringRef Name, StringRef &DemangledName, bool IsCpp) {
   return true;
 }
 
-// DemangledName is a substring of Name. The DemangledName is updated only
-// if true is returned.
-bool isInternalSPIRVBuiltin(StringRef Name, std::string &DemangledName) {
-  if (!Name.startswith("_Z"))
-    return false;
-  constexpr unsigned DemangledNameLenStart = 2;
-  size_t Start = Name.find_first_not_of("0123456789", DemangledNameLenStart);
-  if (!Name.substr(Start).startswith(kSPIRVName::InternalBuiltinPrefix))
-    return false;
-  // LLVM 16 doesn't have itaniumDemangle(const char*, bool) overload
-  // Try using the C-style API first, but it may fail for types like bfloat
-  // (DF16b) that LLVM 16's demangler doesn't recognize
-  char *Demangled =
-      llvm::itaniumDemangle(Name.data(), nullptr, nullptr, nullptr);
-
-  if (Demangled) {
-    // Demangling succeeded - extract function name from demangled string
-    std::string FullDemangled(Demangled);
-    std::free(Demangled);
-    if (!StringRef(FullDemangled).startswith(kSPIRVName::InternalBuiltinPrefix))
-      return false;
-    // Remove the __builtin_spirv_ prefix
-    std::string WithoutPrefix =
-        FullDemangled.substr(strlen(kSPIRVName::InternalBuiltinPrefix));
-    // Extract just the function name (before the opening parenthesis)
-    size_t ParenPos = WithoutPrefix.find('(');
-    if (ParenPos != std::string::npos)
-      DemangledName = WithoutPrefix.substr(0, ParenPos);
-    else
-      DemangledName = WithoutPrefix;
-  } else {
-    // Demangling failed (e.g., for bfloat types in LLVM 16)
-    // Fall back to extracting function name directly from the mangled name
-    // Format: _Z<len>__builtin_spirv_<FunctionName><TypeEncoding>
-    // All conversion function names end with "EXT" or "INTEL"
-    StringRef AfterPrefix =
-        Name.substr(Start + strlen(kSPIRVName::InternalBuiltinPrefix));
-
-    size_t ExtPos = AfterPrefix.find("EXT");
-    size_t IntelPos = AfterPrefix.find("INTEL");
-    size_t FuncNameEnd = StringRef::npos;
-
-    if (ExtPos != StringRef::npos && IntelPos != StringRef::npos)
-      FuncNameEnd =
-          std::min(ExtPos + 3, IntelPos + 5); // +3 for "EXT", +5 for "INTEL"
-    else if (ExtPos != StringRef::npos)
-      FuncNameEnd = ExtPos + 3;
-    else if (IntelPos != StringRef::npos)
-      FuncNameEnd = IntelPos + 5;
-
-    if (FuncNameEnd == StringRef::npos)
-      DemangledName = AfterPrefix.str();
-    else
-      DemangledName = AfterPrefix.substr(0, FuncNameEnd).str();
-  }
-
-  return true;
-}
-
 // Check if a mangled type Name is unsigned
 bool isMangledTypeUnsigned(char Mangled) {
   return Mangled == 'h'    /* uchar */
@@ -698,11 +610,6 @@ ParamType lastFuncParamType(StringRef MangledName) {
   char Mangled = Copy.back();
   std::string Mangled2 = Copy.substr(Copy.size() - 2);
 
-  std::string Mangled5 = Copy.substr(Copy.size() - 5);
-  if (Mangled5 == "DF16b") {
-    return ParamType::FLOAT;
-  }
-
   if (isMangledTypeFP(Mangled) || isMangledTypeHalf(Mangled2)) {
     return ParamType::FLOAT;
   } else if (isMangledTypeUnsigned(Mangled)) {
@@ -868,6 +775,24 @@ void getParameterTypes(Function *F, SmallVectorImpl<StructType *> &ArgTys) {
   free(Buf);
 }
 
+// This is a transitional helper function to fill in mangling information for
+// mangleBuiltin while all the calls to mutateCallInst are being transitioned.
+static void typeMangle(BuiltinFuncMangleInfo *Mangle, ArrayRef<Value *> Args) {
+  if (!Mangle)
+    return;
+  for (unsigned I = 0; I < Args.size(); I++)
+    if (Args[I]->getType()->isPointerTy()) {
+      auto &PointeeTy = Mangle->getTypeMangleInfo(I).PointerElementType;
+      PointeeTy.setPointer(
+          Args[I]->getType()->getNonOpaquePointerElementType());
+      if (PointeeTy.getPointer()->isPointerTy()) {
+        PointeeTy.setPointer(
+            PointeeTy.getPointer()->getNonOpaquePointerElementType());
+        PointeeTy.setInt(true);
+      }
+    }
+}
+
 CallInst *mutateCallInst(
     Module *M, CallInst *CI,
     std::function<std::string(CallInst *, std::vector<Value *> &)> ArgMutate,
@@ -881,6 +806,7 @@ CallInst *mutateCallInst(
     InstName = CI->getName().str();
     CI->setName(InstName + ".old");
   }
+  typeMangle(Mangle, Args);
   auto NewCI = addCallInst(M, NewName, CI->getType(), Args, Attrs, CI, Mangle,
                            InstName, TakeFuncName);
   NewCI->setDebugLoc(CI->getDebugLoc());
@@ -912,6 +838,7 @@ Instruction *mutateCallInst(
   Type *RetTy = CI->getType();
   auto NewName = ArgMutate(CI, Args, RetTy);
   StringRef InstName = CI->getName();
+  typeMangle(Mangle, Args);
   auto NewCI = addCallInst(M, NewName, RetTy, Args, Attrs, CI, Mangle, InstName,
                            TakeFuncName);
   auto NewI = RetMutate(NewCI);
@@ -985,8 +912,16 @@ CallInst *addCallInst(Module *M, StringRef FuncName, Type *RetTy,
 
 CallInst *addCallInstSPIRV(Module *M, StringRef FuncName, Type *RetTy,
                            ArrayRef<Value *> Args, AttributeList *Attrs,
+                           ArrayRef<Type *> PointerElementTypes,
                            Instruction *Pos, StringRef InstName) {
   BuiltinFuncMangleInfo BtnInfo;
+  for (unsigned I = 0; I < PointerElementTypes.size(); I++) {
+    BtnInfo.getTypeMangleInfo(I).PointerElementType.setPointer(
+        PointerElementTypes[I]);
+    if (Args[I]->getType()->isPointerTy())
+      assert(cast<PointerType>(Args[I]->getType())
+                 ->isOpaqueOrPointeeTypeMatches(PointerElementTypes[I]));
+  }
   return addCallInst(M, FuncName, RetTy, Args, Attrs, Pos, &BtnInfo, InstName);
 }
 
@@ -1326,14 +1261,14 @@ static SPIR::RefParamType transTypeDesc(Type *Ty,
     return SPIR::RefParamType(new SPIR::PrimitiveType(SPIR::PRIMITIVE_FLOAT));
   if (Ty->isDoubleTy())
     return SPIR::RefParamType(new SPIR::PrimitiveType(SPIR::PRIMITIVE_DOUBLE));
-  if (Ty->isBFloatTy())
-    return SPIR::RefParamType(new SPIR::PrimitiveType(SPIR::PRIMITIVE_BFLOAT));
   if (auto *VecTy = dyn_cast<FixedVectorType>(Ty)) {
     return SPIR::RefParamType(new SPIR::VectorType(
         transTypeDesc(VecTy->getElementType(), Info), VecTy->getNumElements()));
   }
   if (Ty->isArrayTy()) {
-    return transTypeDesc(PointerType::get(Ty->getArrayElementType(), 0), Info);
+    BuiltinArgTypeMangleInfo DTInfo = Info;
+    DTInfo.PointerElementType.setPointer(Ty->getArrayElementType());
+    return transTypeDesc(Ty->getArrayElementType()->getPointerTo(0), DTInfo);
   }
   if (Ty->isStructTy()) {
     auto Name = Ty->getStructName();
@@ -1361,7 +1296,16 @@ static SPIR::RefParamType transTypeDesc(Type *Ty,
   }
 
   if (Ty->isPointerTy()) {
-    auto ET = Ty->getPointerElementType();
+    auto *ET = Info.PointerElementType.getPointer();
+    if (!ET)
+      ET = Type::getInt8Ty(Ty->getContext());
+    BuiltinArgTypeMangleInfo DTInfo = Info;
+    if (Info.PointerElementType.getInt()) {
+      ET = DTInfo.PointerElementType.getPointer()->getPointerTo(0);
+      DTInfo.PointerElementType.setInt(false);
+    } else {
+      DTInfo.PointerElementType.setPointer(Type::getInt8Ty(Ty->getContext()));
+    }
     SPIR::ParamType *EPT = nullptr;
     if (isa<FunctionType>(ET)) {
       assert(isVoidFuncTy(cast<FunctionType>(ET)) && "Not supported");
@@ -1415,7 +1359,7 @@ static SPIR::RefParamType transTypeDesc(Type *Ty,
 
     if (VoidPtr && ET->isIntegerTy(8))
       ET = Type::getVoidTy(ET->getContext());
-    auto PT = new SPIR::PointerType(transTypeDesc(ET, Info));
+    auto *PT = new SPIR::PointerType(transTypeDesc(ET, DTInfo));
     PT->setAddressSpace(static_cast<SPIR::TypeAttributeEnum>(
         Ty->getPointerAddressSpace() + (unsigned)SPIR::ATTR_ADDR_SPACE_FIRST));
     for (unsigned I = SPIR::ATTR_QUALIFIER_FIRST, E = SPIR::ATTR_QUALIFIER_LAST;
@@ -1932,9 +1876,6 @@ bool checkTypeForSPIRVExtendedInstLowering(IntrinsicInst *II, SPIRVModule *BM) {
       NumElems = VecTy->getNumElements();
       Ty = VecTy->getElementType();
     }
-    if (Ty->isBFloatTy() &&
-        BM->hasCapability(internal::CapabilityBFloat16ArithmeticINTEL))
-      return true;
     if ((!Ty->isFloatTy() && !Ty->isDoubleTy() && !Ty->isHalfTy()) ||
         (!BM->hasCapability(CapabilityVectorAnyINTEL) &&
          ((NumElems > 4) && (NumElems != 8) && (NumElems != 16)))) {
@@ -1951,9 +1892,6 @@ bool checkTypeForSPIRVExtendedInstLowering(IntrinsicInst *II, SPIRVModule *BM) {
       NumElems = VecTy->getNumElements();
       Ty = VecTy->getElementType();
     }
-    if (Ty->isBFloatTy() &&
-        BM->hasCapability(internal::CapabilityBFloat16ArithmeticINTEL))
-      return true;
     if ((!Ty->isIntegerTy()) ||
         (!BM->hasCapability(CapabilityVectorAnyINTEL) &&
          ((NumElems > 4) && (NumElems != 8) && (NumElems != 16)))) {
@@ -2138,7 +2076,7 @@ bool lowerBuiltinVariableToCall(GlobalVariable *GV,
   if (HasIndexArg)
     ArgTy.push_back(Type::getInt32Ty(C));
   std::string MangledName;
-  mangleOpenClBuiltin(FuncName, ArgTy, MangledName);
+  mangleOpenClBuiltin(FuncName, ArgTy, {}, MangledName);
   Function *Func = M->getFunction(MangledName);
   if (!Func) {
     FunctionType *FT = FunctionType::get(ReturnTy, ArgTy, false);
@@ -2170,110 +2108,6 @@ bool lowerBuiltinVariablesToCalls(Module *M) {
   return true;
 }
 
-/// Transforms SPV-IR work-item builtin calls to SPIRV builtin variables.
-/// e.g.
-///  SPV-IR: @_Z33__spirv_BuiltInGlobalInvocationIdi(i)
-///    is transformed as:
-///  x = load GlobalInvocationId; extract x, i
-/// e.g.
-///  SPV-IR: @_Z22__spirv_BuiltInWorkDim()
-///    is transformed as:
-///  load WorkDim
-bool lowerBuiltinCallsToVariables(Module *M) {
-  LLVM_DEBUG(dbgs() << "Enter lowerBuiltinCallsToVariables\n");
-  // Store instructions and functions that need to be removed.
-  SmallVector<Value *, 16> ToRemove;
-  for (auto &F : *M) {
-    // Builtins should be declaration only.
-    if (!F.isDeclaration())
-      continue;
-    StringRef DemangledName;
-    if (!oclIsBuiltin(F.getName(), DemangledName))
-      continue;
-    LLVM_DEBUG(dbgs() << "Function demangled name: " << DemangledName << '\n');
-    SmallVector<StringRef, 2> Postfix;
-    // Deprefix "__spirv_"
-    StringRef Name = dePrefixSPIRVName(DemangledName, Postfix);
-    // Lookup SPIRV Builtin map.
-    if (!SPIRVBuiltInNameMap::rfind(Name.str(), nullptr))
-      continue;
-    std::string BuiltinVarName = DemangledName.str();
-    LLVM_DEBUG(dbgs() << "builtin variable name: " << BuiltinVarName << '\n');
-    bool IsVec = F.getFunctionType()->getNumParams() > 0;
-    Type *GVType =
-        IsVec ? FixedVectorType::get(F.getReturnType(), 3) : F.getReturnType();
-    GlobalVariable *BV = nullptr;
-    // Consider the following LLVM IR:
-    // @__spirv_BuiltInLocalInvocationId = <Global constant>
-    // .....
-    // define spir_kernel void @kernel1(....) {
-    //   %3 = tail call i64 @_Z12get_local_idj(i32 0)
-    //   .....
-    //   return void
-    // }
-    // During the OCLToSPIRV pass, the opencl call will get lowered to
-    // yet another global variable with the name
-    // '@__spirv_BuiltInLocalInvocationId'. In such a case, we would want to
-    // create only a single global variable with this name.
-    if (GlobalVariable *GV = M->getGlobalVariable(BuiltinVarName))
-      BV = GV;
-    else
-      BV = new GlobalVariable(*M, GVType, /*isConstant=*/true,
-                              GlobalValue::ExternalLinkage, nullptr,
-                              BuiltinVarName, 0, GlobalVariable::NotThreadLocal,
-                              SPIRAS_Input);
-    for (auto *U : F.users()) {
-      auto *CI = dyn_cast<CallInst>(U);
-      assert(CI && "invalid instruction");
-      IRBuilder<> Builder(CI);
-      Value *NewValue = Builder.CreateLoad(GVType, BV);
-      LLVM_DEBUG(dbgs() << "Transform: " << *CI << " => " << *NewValue << '\n');
-      if (IsVec) {
-        auto *GVVecTy = cast<FixedVectorType>(GVType);
-        ConstantInt *Bound = Builder.getInt32(GVVecTy->getNumElements());
-        // Create a select on the index first, to avoid undefined behaviour
-        // due to exceeding the vector size by the extractelement.
-        Value *IndexCmp = Builder.CreateICmpULT(CI->getArgOperand(0), Bound);
-        Constant *ZeroIndex =
-            ConstantInt::get(CI->getArgOperand(0)->getType(), 0);
-        Value *ExtractIndex =
-            Builder.CreateSelect(IndexCmp, CI->getArgOperand(0), ZeroIndex);
-
-        // Extract from builtin variable.
-        NewValue = Builder.CreateExtractElement(NewValue, ExtractIndex);
-
-        // Clamp to out-of-range value.
-        Constant *OutOfRangeVal = ConstantInt::get(
-            F.getReturnType(), getBuiltinOutOfRangeValue(BuiltinVarName));
-        NewValue = Builder.CreateSelect(IndexCmp, NewValue, OutOfRangeVal);
-        LLVM_DEBUG(dbgs() << *NewValue << '\n');
-      }
-      NewValue->takeName(CI);
-      CI->replaceAllUsesWith(NewValue);
-      ToRemove.push_back(CI);
-    }
-    ToRemove.push_back(&F);
-  }
-  for (auto *V : ToRemove) {
-    if (auto *I = dyn_cast<Instruction>(V))
-      I->eraseFromParent();
-    else if (auto *F = dyn_cast<Function>(V))
-      F->eraseFromParent();
-    else
-      llvm_unreachable("Unexpected value to remove!");
-  }
-  return true;
-}
-
-bool lowerBuiltins(SPIRVModule *BM, Module *M) {
-  auto Format = BM->getBuiltinFormat();
-  if (Format == BuiltinFormat::Function && !lowerBuiltinVariablesToCalls(M))
-    return false;
-  if (Format == BuiltinFormat::Global && !lowerBuiltinCallsToVariables(M))
-    return false;
-  return true;
-}
-
 bool postProcessBuiltinReturningStruct(Function *F) {
   Module *M = F->getParent();
   LLVMContext *Context = &M->getContext();
@@ -2560,7 +2394,7 @@ public:
       addUnsignedArg(1);
       break;
     case OpImageWrite: {
-      size_t Idx = getImageOperandsIndex(OC);
+      const size_t Idx = getImageOperandsIndex(OC);
       if (Ops.size() > Idx) {
         auto ImOp = static_cast<SPIRVConstant *>(Ops[Idx])->getZExtIntValue();
         if (ImOp & ImageOperandsMask::ImageOperandsZeroExtendMask)
@@ -2643,17 +2477,29 @@ private:
 } // namespace
 
 namespace SPIRV {
-std::string getSPIRVFriendlyIRFunctionName(OCLExtOpKind ExtOpId,
-                                           ArrayRef<Type *> ArgTys,
-                                           Type *RetTy) {
+void BuiltinFuncMangleInfo::fillPointerElementTypes(
+    ArrayRef<PointerIndirectPair> PointerElementTys) {
+  for (unsigned I = 0; I < PointerElementTys.size(); I++) {
+    getTypeMangleInfo(I).PointerElementType = PointerElementTys[I];
+  }
+}
+
+std::string
+getSPIRVFriendlyIRFunctionName(OCLExtOpKind ExtOpId, ArrayRef<Type *> ArgTys,
+                               ArrayRef<PointerIndirectPair> PointerElementTys,
+                               Type *RetTy) {
   OpenCLStdToSPIRVFriendlyIRMangleInfo MangleInfo(ExtOpId, ArgTys, RetTy);
+  MangleInfo.fillPointerElementTypes(PointerElementTys);
   return mangleBuiltin(MangleInfo.getUnmangledName(), ArgTys, &MangleInfo);
 }
 
-std::string getSPIRVFriendlyIRFunctionName(const std::string &UniqName,
-                                           spv::Op OC, ArrayRef<Type *> ArgTys,
-                                           ArrayRef<SPIRVValue *> Ops) {
+std::string
+getSPIRVFriendlyIRFunctionName(const std::string &UniqName, spv::Op OC,
+                               ArrayRef<Type *> ArgTys,
+                               ArrayRef<PointerIndirectPair> PointerElementTys,
+                               ArrayRef<SPIRVValue *> Ops) {
   SPIRVFriendlyIRMangleInfo MangleInfo(OC, ArgTys, Ops);
+  MangleInfo.fillPointerElementTypes(PointerElementTys);
   return mangleBuiltin(UniqName, ArgTys, &MangleInfo);
 }
 
