@@ -279,8 +279,7 @@ public:
   template <class T> T *addType(T *Ty);
   SPIRVTypeArray *addArrayType(SPIRVType *, SPIRVConstant *) override;
   SPIRVTypeBool *addBoolType() override;
-  SPIRVTypeFloat *addFloatType(unsigned BitWidth,
-                               unsigned FloatingPointEncoding) override;
+  SPIRVTypeFloat *addFloatType(unsigned BitWidth) override;
   SPIRVTypeFunction *addFunctionType(SPIRVType *,
                                      const std::vector<SPIRVType *> &) override;
   SPIRVTypeInt *addIntegerType(unsigned BitWidth) override;
@@ -592,13 +591,7 @@ private:
   SPIRVCapMap CapMap;
   SPIRVUnknownStructFieldMap UnknownStructFieldMap;
   std::map<unsigned, SPIRVTypeInt *> IntTypeMap;
-  SmallDenseMap<std::pair<unsigned, unsigned>, SPIRVTypeFloat *, 4>
-      FloatTypeMap;
   std::map<unsigned, SPIRVConstant *> LiteralMap;
-  SmallDenseMap<std::pair<unsigned, SPIRVType *>, SPIRVTypePointer *, 4>
-      PointerTypeMap;
-  SmallDenseMap<std::pair<SPIRVType *, SPIRVWord>, SPIRVTypeVector *, 4>
-      VectorTypeMap;
   std::vector<SPIRVExtInst *> DebugInstVec;
   std::vector<SPIRVExtInst *> AuxDataInstVec;
   std::vector<SPIRVModuleProcessed *> ModuleProcessedVec;
@@ -1011,15 +1004,9 @@ SPIRVTypeInt *SPIRVModuleImpl::addIntegerType(unsigned BitWidth) {
   return addType(Ty);
 }
 
-SPIRVTypeFloat *SPIRVModuleImpl::addFloatType(unsigned BitWidth,
-                                              unsigned FloatingPointEncoding) {
-  auto Desc = std::make_pair(BitWidth, FloatingPointEncoding);
-  auto Loc = FloatTypeMap.find(Desc);
-  if (Loc != FloatTypeMap.end())
-    return Loc->second;
-  auto *Ty = new SPIRVTypeFloat(this, getId(), BitWidth, FloatingPointEncoding);
-  FloatTypeMap[Desc] = Ty;
-  return addType(Ty);
+SPIRVTypeFloat *SPIRVModuleImpl::addFloatType(unsigned BitWidth) {
+  SPIRVTypeFloat *T = addType(new SPIRVTypeFloat(this, getId(), BitWidth));
+  return T;
 }
 
 SPIRVTypePointer *
@@ -1056,13 +1043,7 @@ void SPIRVModuleImpl::closeStructType(SPIRVTypeStruct *T, bool Packed) {
 
 SPIRVTypeVector *SPIRVModuleImpl::addVectorType(SPIRVType *CompType,
                                                 SPIRVWord CompCount) {
-  auto Desc = std::make_pair(CompType, CompCount);
-  auto Loc = VectorTypeMap.find(Desc);
-  if (Loc != VectorTypeMap.end())
-    return Loc->second;
-  auto *Ty = new SPIRVTypeVector(this, getId(), CompType, CompCount);
-  VectorTypeMap[Desc] = Ty;
-  return addType(Ty);
+  return addType(new SPIRVTypeVector(this, getId(), CompType, CompCount));
 }
 
 SPIRVTypeJointMatrixINTEL *
@@ -1553,8 +1534,6 @@ SPIRVInstruction *SPIRVModuleImpl::addBinaryInst(Op TheOpCode, SPIRVType *Type,
                                                  SPIRVValue *Op1,
                                                  SPIRVValue *Op2,
                                                  SPIRVBasicBlock *BB) {
-  if (Type->isTypeFloat(16, FPEncodingBFloat16KHR) && TheOpCode != OpDot)
-    addCapability(internal::CapabilityBFloat16ArithmeticINTEL);
   return addInstruction(SPIRVInstTemplateBase::create(
                             TheOpCode, Type, getId(),
                             getVec(Op1->getId(), Op2->getId()), BB, this),
@@ -1578,8 +1557,6 @@ SPIRVInstruction *SPIRVModuleImpl::addUnaryInst(Op TheOpCode,
                                                 SPIRVType *TheType,
                                                 SPIRVValue *Op,
                                                 SPIRVBasicBlock *BB) {
-  if (TheType->isTypeFloat(16, FPEncodingBFloat16KHR) && TheOpCode != OpDot)
-    addCapability(internal::CapabilityBFloat16ArithmeticINTEL);
   return addInstruction(
       SPIRVInstTemplateBase::create(TheOpCode, TheType, getId(),
                                     getVec(Op->getId()), BB, this),
