@@ -72,16 +72,9 @@ enum InternalOp {
   IOpCooperativeMatrixLoadCheckedINTEL = 6193,
   IOpCooperativeMatrixStoreCheckedINTEL = 6194,
   IOpCooperativeMatrixConstructCheckedINTEL = 6195,
-  IOpClampConvertFToFINTEL = 6216,
-  IOpStochasticRoundFToFINTEL = 6217,
-  IOpClampStochasticRoundFToFINTEL = 6218,
-  IOpClampStochasticRoundFToSINTEL = 6219,
-  IOpPredicatedLoadINTEL = 6258,
-  IOpPredicatedStoreINTEL = 6259,
   IOpJointMatrixWorkItemLengthINTEL = 6410,
   IOpComplexFMulINTEL = 6415,
   IOpComplexFDivINTEL = 6416,
-  IOpClampConvertFToSINTEL = 6424,
   IOpRoundFToTF32INTEL = 6426,
   IOpMaskedGatherINTEL = 6428,
   IOpMaskedScatterINTEL = 6429,
@@ -90,7 +83,6 @@ enum InternalOp {
   IOpConvertHandleToImageINTEL = 6529,
   IOpConvertHandleToSamplerINTEL = 6530,
   IOpConvertHandleToSampledImageINTEL = 6531,
-  IOpFSigmoidINTEL = 6168,
   IOpPrev = OpMax - 2,
   IOpForward
 };
@@ -114,24 +106,15 @@ enum InternalCapability {
   ICapFPGADSPControlINTEL = 5908,
   ICapFPGAInvocationPipeliningAttributesINTEL = 5916,
   ICapRuntimeAlignedAttributeINTEL = 5939,
+  ICapFastCompositeINTEL = 6093,
+  ICapOptNoneINTEL = 6094,
   ICapTokenTypeINTEL = 6112,
   ICapBfloat16ConversionINTEL = 6115,
   ICapabilityJointMatrixINTEL = 6118,
   ICapabilityHWThreadQueryINTEL = 6134,
   ICapFPArithmeticFenceINTEL = 6144,
   ICapGlobalVariableDecorationsINTEL = 6146,
-  ICapabilitySigmoidINTEL = 6167,
   ICapabilityCooperativeMatrixCheckedInstructionsINTEL = 6192,
-  ICapabilityFloat4E2M1INTEL = 6212,
-  ICapabilityFloat4E2M1CooperativeMatrixINTEL = 6213,
-  ICapabilityFloatConversionsINTEL = 6215,
-  ICapabilityBFloat16ArithmeticINTEL = 6226,
-  ICapabilityAtomicBFloat16AddINTEL = 6255,
-  ICapabilityAtomicBFloat16MinMaxINTEL = 6256,
-  ICapabilityPredicatedIOINTEL = 6257,
-  ICapabilityAtomicInt16CompareExchangeINTEL = 6260,
-  ICapabilityInt16AtomicsINTEL = 6261,
-  ICapabilityAtomicBFloat16LoadStoreINTEL = 6262,
   ICapabilityCooperativeMatrixPrefetchINTEL = 6411,
   ICapabilityComplexFloatMulDivINTEL = 6414,
   ICapabilityTensorFloat32RoundingINTEL = 6425,
@@ -142,17 +125,17 @@ enum InternalCapability {
   ICapabilityBindlessImagesINTEL = 6528
 };
 
+enum InternalFunctionControlMask { IFunctionControlOptNoneINTELMask = 0x10000 };
+
 enum InternalExecutionMode {
+  IExecModeFastCompositeKernelINTEL = 6088,
   IExecModeStreamingInterfaceINTEL = 6154,
   IExecModeMaximumRegistersINTEL = 6461,
   IExecModeMaximumRegistersIdINTEL = 6462,
   IExecModeNamedMaximumRegistersINTEL = 6463
 };
 
-enum InternalLoopControlMask {
-  ILoopControlLoopCountINTELMask = 0x1000000,
-  ILoopControlMaxReinvocationDelayINTELMask = 0x2000000
-};
+enum InternalLoopControlMask { ILoopControlLoopCountINTELMask = 0x1000000 };
 
 constexpr LinkageType LinkageTypeInternal =
     static_cast<LinkageType>(ILTInternal);
@@ -166,11 +149,6 @@ enum InternalJointMatrixLayout {
 
 enum InternalJointMatrixUse { MatrixA = 0, MatrixB = 1, Accumulator = 2 };
 
-enum InternalFPEncoding {
-  FPEncodingFloat4E2M1INTEL = 6214,
-  FPEncodingMax = 0x7fffffff,
-};
-
 enum InternalBuiltIn {
   IBuiltInSubDeviceIDINTEL = 6135,
   IBuiltInGlobalHWThreadIDINTEL = 6136,
@@ -237,25 +215,6 @@ _SPIRV_OP(Capability, BindlessImagesINTEL)
 _SPIRV_OP(Op, ConvertHandleToImageINTEL)
 _SPIRV_OP(Op, ConvertHandleToSamplerINTEL)
 _SPIRV_OP(Op, ConvertHandleToSampledImageINTEL)
-
-_SPIRV_OP(Capability, AtomicBFloat16AddINTEL)
-_SPIRV_OP(Capability, AtomicBFloat16MinMaxINTEL)
-
-_SPIRV_OP(Capability, PredicatedIOINTEL)
-_SPIRV_OP(Op, PredicatedLoadINTEL)
-_SPIRV_OP(Op, PredicatedStoreINTEL)
-
-_SPIRV_OP(Capability, SigmoidINTEL)
-_SPIRV_OP(Op, FSigmoidINTEL)
-_SPIRV_OP(Capability, Float4E2M1INTEL)
-_SPIRV_OP(Capability, Float4E2M1CooperativeMatrixINTEL)
-
-_SPIRV_OP(Capability, FloatConversionsINTEL)
-_SPIRV_OP(Op, ClampConvertFToFINTEL)
-_SPIRV_OP(Op, ClampConvertFToSINTEL)
-_SPIRV_OP(Op, StochasticRoundFToFINTEL)
-_SPIRV_OP(Op, ClampStochasticRoundFToFINTEL)
-_SPIRV_OP(Op, ClampStochasticRoundFToSINTEL)
 #undef _SPIRV_OP
 
 constexpr SourceLanguage SourceLanguagePython =
@@ -311,6 +270,8 @@ constexpr Decoration DecorationMaxConcurrencyINTEL =
     static_cast<Decoration>(IDecMaxConcurrencyINTEL);
 constexpr Decoration DecorationPipelineEnableINTEL =
     static_cast<Decoration>(IDecPipelineEnableINTEL);
+constexpr Decoration DecorationCallableFunctionINTEL =
+    static_cast<Decoration>(IDecCallableFunctionINTEL);
 constexpr Decoration DecorationRuntimeAlignedINTEL =
     static_cast<Decoration>(IDecRuntimeAlignedINTEL);
 constexpr Decoration DecorationHostAccessINTEL =
@@ -326,6 +287,10 @@ constexpr Decoration DecorationCacheControlLoadINTEL =
 constexpr Decoration DecorationCacheControlStoreINTEL =
     static_cast<Decoration>(IDecCacheControlStoreINTEL);
 
+constexpr Capability CapabilityFastCompositeINTEL =
+    static_cast<Capability>(ICapFastCompositeINTEL);
+constexpr Capability CapabilityOptNoneINTEL =
+    static_cast<Capability>(ICapOptNoneINTEL);
 constexpr Capability CapabilityFPGADSPControlINTEL =
     static_cast<Capability>(ICapFPGADSPControlINTEL);
 constexpr Capability CapabilityFPGAInvocationPipeliningAttributesINTEL =
@@ -342,25 +307,20 @@ constexpr Capability CapabilityGlobalVariableDecorationsINTEL =
     static_cast<Capability>(ICapGlobalVariableDecorationsINTEL);
 constexpr Capability CapabilityRegisterLimitsINTEL =
     static_cast<Capability>(ICapRegisterLimitsINTEL);
-constexpr Capability CapabilityBFloat16ArithmeticINTEL =
-    static_cast<Capability>(ICapabilityBFloat16ArithmeticINTEL);
-constexpr Capability CapabilityAtomicInt16CompareExchangeINTEL =
-    static_cast<Capability>(ICapabilityAtomicInt16CompareExchangeINTEL);
-constexpr Capability CapabilityInt16AtomicsINTEL =
-    static_cast<Capability>(ICapabilityInt16AtomicsINTEL);
-constexpr Capability CapabilityAtomicBFloat16LoadStoreINTEL =
-    static_cast<Capability>(ICapabilityAtomicBFloat16LoadStoreINTEL);
+
+constexpr FunctionControlMask FunctionControlOptNoneINTELMask =
+    static_cast<FunctionControlMask>(IFunctionControlOptNoneINTELMask);
 
 constexpr Decoration DecorationMathOpDSPModeINTEL =
     static_cast<Decoration>(IDecMathOpDSPModeINTEL);
 
+constexpr ExecutionMode ExecutionModeFastCompositeKernelINTEL =
+    static_cast<ExecutionMode>(IExecModeFastCompositeKernelINTEL);
 constexpr ExecutionMode ExecutionModeStreamingInterfaceINTEL =
     static_cast<ExecutionMode>(IExecModeStreamingInterfaceINTEL);
 
 constexpr LoopControlMask LoopControlLoopCountINTELMask =
     static_cast<LoopControlMask>(ILoopControlLoopCountINTELMask);
-static const LoopControlMask LoopControlMaxReinvocationDelayINTELMask =
-    static_cast<LoopControlMask>(ILoopControlMaxReinvocationDelayINTELMask);
 constexpr ExecutionMode ExecutionModeMaximumRegistersINTEL =
     static_cast<ExecutionMode>(IExecModeMaximumRegistersINTEL);
 constexpr ExecutionMode ExecutionModeMaximumRegistersIdINTEL =
