@@ -47,6 +47,7 @@
 
 #include "llvm/Bitcode/BitcodeReader.h"
 #include "llvm/Bitcode/BitcodeWriter.h"
+#include "llvm/IR/Constants.h"
 #include "llvm/IR/LLVMContext.h"
 #include "llvm/IR/Module.h"
 #include "llvm/IR/Verifier.h"
@@ -154,6 +155,12 @@ static cl::opt<bool>
     SPIRVToolsDis("spirv-tools-dis", cl::init(false),
                   cl::desc("Emit textual assembly using SPIRV-Tools"));
 
+static cl::opt<bool>
+    EmitOpaquePointers("emit-opaque-pointers", cl::init(false),
+                       cl::desc("Emit opaque instead of typed LLVM pointers "
+                                "for the translation from SPIR-V."),
+                       cl::Hidden);
+
 static cl::opt<bool> SPIRVEmitFunctionPtrAddrSpace(
     "spirv-emit-function-ptr-addr-space", cl::init(false),
     cl::desc("Emit and consume CodeSectionINTEL for function pointers"));
@@ -256,16 +263,6 @@ static cl::opt<bool> SPIRVReplaceLLVMFmulAddWithOpenCLMad(
              "instruction from OpenCL extended instruction set"),
     cl::init(true));
 
-static cl::opt<SPIRV::BuiltinFormat> SPIRVBuiltinFormat(
-    "spirv-builtin-format",
-    cl::desc("Set LLVM-IR representation of SPIR-V builtin variables:"),
-    cl::init(SPIRV::BuiltinFormat::Function),
-    cl::values(
-        clEnumValN(SPIRV::BuiltinFormat::Function, "function",
-                   "Use functions to represent SPIR-V builtin variables"),
-        clEnumValN(SPIRV::BuiltinFormat::Global, "global",
-                   "Use globals to represent SPIR-V builtin variables")));
-
 static std::string removeExt(const std::string &FileName) {
   size_t Pos = FileName.find_last_of(".");
   if (Pos != std::string::npos)
@@ -383,6 +380,8 @@ static bool isFileEmpty(const std::string &FileName) {
 
 static int convertSPIRVToLLVM(const SPIRV::TranslatorOpts &Opts) {
   LLVMContext Context;
+  Context.setOpaquePointers(EmitOpaquePointers);
+
   std::ifstream IFS(InputFile, std::ios::binary);
   Module *M;
   std::string Err;
@@ -710,15 +709,6 @@ int main(int Ac, char **Av) {
 
   Opts.setFPContractMode(FPCMode);
 
-  if (SPIRVBuiltinFormat.getNumOccurrences() != 0) {
-    if (!IsReverse) {
-      errs() << "Note: --spirv-builtin-format option ignored as it only "
-                "affects translation from SPIR-V to LLVM IR";
-    } else {
-      Opts.setBuiltinFormat(SPIRVBuiltinFormat);
-    }
-  }
-
   if (SPIRVMemToReg)
     Opts.setMemToRegEnabled(SPIRVMemToReg);
   if (SPIRVGenKernelArgNameMD)
@@ -837,7 +827,7 @@ int main(int Ac, char **Av) {
     }
 
     SPIRV::SPIRVModuleTextReport TextReport =
-        SPIRV::formatSpirvReport(*BinReport);
+        SPIRV::formatSpirvReport(BinReport.value());
 
     std::cout << "SPIR-V module report:"
               << "\n Version: " << TextReport.Version
